<?php
// +----------------------------------------------------------------------+
// | Anuko Time Tracker
// +----------------------------------------------------------------------+
// | Copyright (c) Anuko International Ltd. (http://www.anuko.com)
// +----------------------------------------------------------------------+
// | LIBERAL FREEWARE LICENSE: This source code document may be used
// | by anyone for any purpose, and freely redistributed alone or in
// | combination with other software, provided that the license is obeyed.
// |
// | There are only two ways to violate the license:
// |
// | 1. To redistribute this code in source form, with the copyright
// |    notice or license removed or altered. (Distributing in compiled
// |    forms without embedded copyright notices is permitted).
// |
// | 2. To redistribute modified versions of this code in *any* form
// |    that bears insufficient indications that the modifications are
// |    not the work of the original author(s).
// |
// | This license applies to this document only, not any other software
// | that it may be combined with.
// |
// +----------------------------------------------------------------------+
// | Contributors:
// | http://www.anuko.com/time_tracker/credits.htm
// +----------------------------------------------------------------------+

import('ttClientHelper');
import('DateAndTime');
import('Period');
import('ttTimeHelper');

// Class ttReportHelper is used for help with reports.
class ttReportHelper {

  // getWhere prepares WHERE clause for report query.
  static function getWhere($bean) {
    global $user;

  	// Prepare dropdown parts.
    $dropdown_parts = '';
    if ($bean->getAttribute('client')) $dropdown_parts .= ' and l.client_id = '.$bean->getAttribute('client');
    if ($bean->getAttribute('option')) $dropdown_parts .= ' and l.id in(select log_id from tt_custom_field_log where option_id = '.$bean->getAttribute('option').')';
    if ($bean->getAttribute('project')) $dropdown_parts .= ' and l.project_id = '.$bean->getAttribute('project');
    if ($bean->getAttribute('task')) $dropdown_parts .= ' and l.task_id = '.$bean->getAttribute('task');
    if ($bean->getAttribute('include_records')=='1') $dropdown_parts .= ' and l.billable = 1';
    if ($bean->getAttribute('include_records')=='2') $dropdown_parts .= ' and l.billable = 0';
        
    // Prepare user list part.
    $userlist = -1;
    if ($user->canManageTeam() && is_array($bean->getAttribute('users')))
      $userlist = join(',', $bean->getAttribute('users'));
    // Prepare sql query part for user list.
    $user_list_part = null;
    if ($user->canManageTeam())
      $user_list_part = " and l.user_id in ($userlist)";
    else
      $user_list_part = " and l.user_id = ".$user->id;
    
    // Prepare sql query part for where.
    if ($bean->getAttribute('period'))
      $period = new Period($bean->getAttribute('period'), new DateAndTime($user->date_format));
    else {
      $period = new Period();
      $period->setPeriod(
        new DateAndTime($user->date_format, $bean->getAttribute('start_date')),
        new DateAndTime($user->date_format, $bean->getAttribute('end_date')));
    }
    $where = " where l.date >= '".$period->getBeginDate(DB_DATEFORMAT)."' and l.date <= '".$period->getEndDate(DB_DATEFORMAT)."'".
      " $user_list_part $dropdown_parts";
    return $where;
  }
	
  // The getItems retrieves tt_log items associated with a report.
  // Report properties are set in session bean (passed as parameter).
  static function getItems($bean) {
    global $user;
    global $i18n;
    $mdb2 = getConnection();
    
    $group_by_option = $bean->getAttribute('group_by');
    $convertTo12Hour = ('%I:%M %p' == $user->time_format) && ($bean->getAttribute('chstart') || $bean->getAttribute('chfinish')); 
    
    $fields = array(); // An array of fields for database query.
    array_push($fields, 'l.id as id');
    array_push($fields, 'l.date as date');
    if($user->canManageTeam())
      array_push($fields, 'u.name as user');
    // Add client name if it is selected.
    if ($bean->getAttribute('chclient') || 'client' == $group_by_option)
      array_push($fields, 'c.name as client');
    // Add project name if it is selected.
    if ($bean->getAttribute('chproject') || 'project' == $group_by_option)
      array_push($fields, 'p.name as project');
    // Add task name if it is selected.
    if ($bean->getAttribute('chtask') || 'task' == $group_by_option)
      array_push($fields, 't.name as task');
    // Add custom field.
    $include_cf_1 = $bean->getAttribute('chcf_1') || 'cf_1' == $group_by_option;
    if ($include_cf_1) {
      require_once('plugins/CustomFields.class.php');
      $custom_fields = new CustomFields($user->team_id);
      $cf_1_type = $custom_fields->fields[0]['type'];
      if ($cf_1_type == CustomFields::TYPE_TEXT) {
        array_push($fields, 'cfl.value as cf_1');	
      } else if ($cf_1_type == CustomFields::TYPE_DROPDOWN) {
  	    array_push($fields, 'cfo.value as cf_1');	
      }
    }
    // Add start time.
    if ($bean->getAttribute('chstart'))
      array_push($fields, "TIME_FORMAT(l.start, '%k:%i') as start");
    // Add finish time.
    if ($bean->getAttribute('chfinish'))
      array_push($fields, "TIME_FORMAT(sec_to_time(time_to_sec(l.start) + time_to_sec(l.duration)), '%k:%i') as finish");
    // Add duration.
    if ($bean->getAttribute('chduration'))
      array_push($fields, "TIME_FORMAT(l.duration, '%k:%i') as duration");
    // Add note.
    if ($bean->getAttribute('chnote'))
      array_push($fields, 'l.comment as note');
    // Handle cost.
    $includeCost = $bean->getAttribute('chcost');
    if ($includeCost) {
      if (MODE_TIME == $user->tracking_mode)
        array_push($fields, "round(l.billable * coalesce(u.rate, 0) * time_to_sec(l.duration)/3600, 2) as cost");   // Use default user rate.
      else
        array_push($fields, "round(l.billable * coalesce(upb.rate, 0) * time_to_sec(l.duration)/3600, 2) as cost"); // Use project rate for user.
    }
    // Add invoice name if it is selected.
    if ($user->canManageTeam() && $bean->getAttribute('chinvoice'))
      array_push($fields, 'i.name as invoice');

    // Prepare sql query part for left joins.
    $left_joins = null;
    if ($bean->getAttribute('chclient') || 'client' == $group_by_option)
      $left_joins .= " left join tt_clients c on (c.id = l.client_id)";
    if ($user->canManageTeam() && $bean->getAttribute('chinvoice'))
      $left_joins .= " left join tt_invoices i on (i.id = l.invoice_id)";
    if ($user->canManageTeam())
       $left_joins .= " left join tt_users u on (u.id = l.user_id)";
    if ($bean->getAttribute('chproject') || 'project' == $group_by_option)
      $left_joins .= " left join tt_projects p on (p.id = l.project_id)";	
    if ($bean->getAttribute('chtask') || 'task' == $group_by_option)
      $left_joins .= " left join tt_tasks t on (t.id = l.task_id)";		
    if ($include_cf_1) {
      if ($cf_1_type == CustomFields::TYPE_TEXT)
        $left_joins .= " left join tt_custom_field_log cfl on (l.id = cfl.log_id)";
      else if ($cf_1_type == CustomFields::TYPE_DROPDOWN) {
        $left_joins .=  " left join tt_custom_field_log cfl on (l.id = cfl.log_id)".
          " left join tt_custom_field_options cfo on (cfl.option_id = cfo.id)";
      }
    }
    if ($includeCost && MODE_TIME != $user->tracking_mode)
      $left_joins .= " left join tt_user_project_binds upb on (l.user_id = upb.user_id and l.project_id = upb.project_id)";
    
    $where = ttReportHelper::getWhere($bean);
    
    // Determine sort part.
    $sort_part = 'order by ';
    if ('no_grouping' != $group_by_option)
      $sort_part .= $group_by_option.', ';
    $sort_part .= 'date';
    if ($bean->getAttribute('chstart'))
      $sort_part .= ', start';
    $sort_part .= ', id';
    
    // Obtain regular rows for report.
    $sql = "select ".join(', ', $fields)." from tt_log l $left_joins $where $sort_part";
    $res = $mdb2->query($sql);
    if (!is_a($res, 'PEAR_Error')) {
      while ($val = $res->fetchRow()) {
          if ($convertTo12Hour) {
            if($val['start'] != '')
              $val['start'] = ttTimeHelper::to12HourFormat($val['start']);
            if($val['finish'] != '')
              $val['finish'] = ttTimeHelper::to12HourFormat($val['finish']);
          }
  	      if (isset($val['cost'])) {
  	        if ('.' != $i18n->getKey('format.float_delimiter'))
  	          $val['cost'] = str_replace('.', $i18n->getKey('format.float_delimiter'), $val['cost']);
  	      }
  	      if ('no_grouping' != $group_by_option)
            $val['grouped_by'] = $val[$group_by_option];
          $row = $val;
          $report_items[] = $row;
        }
      } else
  	    die($res->getMessage());

    return $report_items;
  }
  
  // getTotals calculates total hours and cost for report items.
  static function getTotals($bean)
  {
  	global $user;
  	global $i18n;
    $mdb2 = getConnection();
    
    $where = ttReportHelper::getWhere($bean);
  	
    // Calculate totals for report summary.
    $total_time = 0;
    $total_cost = 0;
    
    $includeCost = $user->canManageTeam() && $bean->getAttribute('chcost');
     if ($includeCost) {
      if (MODE_TIME == $user->tracking_mode) {
        $sql = "select sum(time_to_sec(l.duration)) as time, sum(round(l.billable * coalesce(u.rate, 0) * time_to_sec(l.duration)/3600, 2)) as cost 
          from tt_log l
          left join tt_users u on (l.user_id = u.id) $where";
      } else {
        $sql = "select sum(time_to_sec(l.duration)) as time, sum(round(l.billable * coalesce(upb.rate, 0) * time_to_sec(l.duration)/3600, 2)) as cost 
          from tt_log l
          left join tt_user_project_binds upb on (l.user_id = upb.user_id and l.project_id = upb.project_id) $where";  	
      }
    } else
      $sql = "select sum(time_to_sec(l.duration)) as time from tt_log l $where";

    $res = $mdb2->query($sql);
    if (!is_a($res, 'PEAR_Error')) {
      $val = $res->fetchRow();
      $total_time = sec_to_time_fmt_hm($val['time']);
      if ($includeCost) {
      	$total_cost = $val['cost'];
      	if (!$total_cost) $total_cost = '0.00';
      	if ('.' != $i18n->getKey('format.float_delimiter'))
  	      $total_cost = str_replace('.', $i18n->getKey('format.float_delimiter'), $total_cost);
      }
    } else
      die($res->getMessage());

    if ($bean->getAttribute('period'))
      $period = new Period($bean->getAttribute('period'), new DateAndTime($user->date_format));
    else {
      $period = new Period();
      $period->setPeriod(
        new DateAndTime($user->date_format, $bean->getAttribute('start_date')),
        new DateAndTime($user->date_format, $bean->getAttribute('end_date')));
    }

    $totals['start_date'] = $period->getBeginDate();
  	$totals['end_date'] = $period->getEndDate();
  	$totals['time'] = $total_time;
  	$totals['cost'] = $total_cost;
 	
  	return $totals;
  }
  
  // getSubtotals calculates subtotals when a report is grouped by.
  static function getSubtotals($bean) {
  	global $user;
  	global $i18n;
  	
  	$subtotals = array();
    $group_by_option = $bean->getAttribute('group_by');
    if ('no_grouping' == $group_by_option) return $subtotals;
    
    $mdb2 = getConnection();
    
    // Determine group by field and a required join.
    switch ($group_by_option) {
  	  case 'date':
        $group_field = 'l.date';
        $group_join = '';
        break;
      case 'user':
        $group_field = 'u.name';
        $group_join = 'left join tt_users u on (l.user_id = u.id) ';
        break;
      case 'client':
        $group_field = 'c.name';
        $group_join = 'left join tt_clients c on (l.client_id = c.id) ';
        break;
      case 'project':
  	    $group_field = 'p.name';
        $group_join = 'left join tt_projects p on (l.project_id = p.id) ';
  	    break;
      case 'task':
  	    $group_field = 't.name';
        $group_join = 'left join tt_tasks t on (l.task_id = t.id) ';
  	    break;
      case 'cf_1':
  	    require_once('plugins/CustomFields.class.php');
  	    $group_field = 'cfo.value';
  	    $custom_fields = new CustomFields($user->team_id);
  	    if ($custom_fields->fields[0]['type'] == CustomFields::TYPE_TEXT)
          $group_join = 'left join tt_custom_field_log cfl on (l.id = cfl.log_id) left join tt_custom_field_options cfo on (cfl.value = cfo.id) ';
        else if ($custom_fields->fields[0]['type'] == CustomFields::TYPE_DROPDOWN)
          $group_join = 'left join tt_custom_field_log cfl on (l.id = cfl.log_id) left join tt_custom_field_options cfo on (cfl.option_id = cfo.id) ';
    	  break;
    }

    $where = ttReportHelper::getWhere($bean);
    $includeCost = $bean->getAttribute('chcost');
    if ($includeCost) {
      if (MODE_TIME == $user->tracking_mode) {
        $sql = "select $group_field as group_field, sum(time_to_sec(l.duration)) as time, 
          sum(round(l.billable * coalesce(u.rate, 0) * time_to_sec(l.duration)/3600, 2)) as cost from tt_log l
          left join tt_users u on (l.user_id = u.id)
          $group_join $where group by $group_field";      	
      } else {
        // If we are including cost and tracking projects, our query (the same as above) needs to join the tt_user_project_binds table.
        $sql = "select $group_field as group_field, sum(time_to_sec(l.duration)) as time, 
          sum(round(l.billable * coalesce(upb.rate, 0) * time_to_sec(l.duration)/3600, 2)) as cost from tt_log l 
          $group_join
          left join tt_user_project_binds upb on (l.user_id = upb.user_id and l.project_id = upb.project_id) $where group by $group_field";
      }
    } else {
  	   $sql = "select $group_field as group_field, sum(time_to_sec(l.duration)) as time from tt_log l 
         $group_join $where group by $group_field";
    }
    $res = $mdb2->query($sql);
    if (!is_a($res, 'PEAR_Error')) {
  	  while ($val = $res->fetchRow()) {
        if ($includeCost) {
  	      if ('.' != $i18n->getKey('format.float_delimiter'))
  	        $val['cost'] = str_replace('.', $i18n->getKey('format.float_delimiter'), $val['cost']);
      	  $subtotals[$val['group_field']] = array('name'=>$val['group_field'],'time'=>sec_to_time_fmt_hm($val['time']),'cost'=>$val['cost']);
        } else
          $subtotals[$val['group_field']] = array('name'=>$val['group_field'],'time'=>sec_to_time_fmt_hm($val['time']));
      }
    } else
      die($res->getMessage());

    return $subtotals;
  }
  
  // The assignToInvoice assigns a set of records to a specific invoice.
  static function assignToInvoice($record_ids, $invoice_id)
  {
  	$mdb2 = getConnection();
    $sql = "update tt_log set invoice_id = ".$mdb2->quote($invoice_id).
      " where id in(".join(', ', $record_ids).")";
    $res = $mdb2->query($sql);
    if (is_a($res, 'PEAR_Error'))
  	  die($res->getMessage());
  }

  // prepareInvoiceBody - prepares an email body for report.
  static function prepareReportBody($bean, $comment)
  {
  	global $user;
  	global $i18n;
  	
  	$report_items = ttReportHelper::getItems($bean);
  	$subtotals = ttReportHelper::getSubtotals($bean);
    $totals = ttReportHelper::getTotals($bean);
    
    // Use custom fields plugin if it is enabled.
    if (in_array('cf', explode(',', $user->plugins))) {
      require_once('plugins/CustomFields.class.php');
      $custom_fields = new CustomFields($user->team_id);
    }

    // Define some styles to use in email.
    $tableHeader = 'font-weight: bold; background-color: #a6ccf7; text-align: center;';
    $rowItem = 'background-color: #f9f9f9;';
    $rowSubtotal = 'background-color: #e0e0e0;';
    $cellLeftAligned = 'text-align: left; vertical-align: top;';
    $cellRightAligned = 'text-align: right; vertical-align: top;';
    $cellLeftAlignedSubtotal = 'font-weight: bold; text-align: left; vertical-align: top;';
    $cellRightAlignedSubtotal = 'font-weight: bold; text-align: right; vertical-align: top;';
    
    // Start creating email body.
    $body = '<html>';
    $body .= '<head><meta http-equiv="content-type" content="text/html; charset='.CHARSET.'"></head>';
    $body .= '<body>';
    
    $group_by = $bean->getAttribute('group_by');
    if ('no_grouping' != $group_by && $bean->getAttribute('chtotalsonly')) {
      // Totals only report. Output subtotals.
      
      // Determine group_by header.
      if ('cf_1' == $group_by)
        $group_by_header = $custom_fields->fields[0]['label'];
      else {
        $key = 'label.'.$group_by;
        $group_by_header = $i18n->getKey($key);
      }
    	
      $body .= '<table border="0" cellpadding="4" cellspacing="0" width="100%">';
      $body .= '<tr>';
      $body .= '<td style="'.$tableHeader.'">'.$group_by_header.'</td>';
      if ($bean->getAttribute('chduration'))
        $body .= '<td style="'.$tableHeader.'">'.$i18n->getKey('label.duration').'</td>';
      if ($bean->getAttribute('chcost'))
        $body .= '<td style="'.$tableHeader.'">'.$i18n->getKey('label.cost').'</td>';
      $body .= '</tr>';
      foreach($subtotals as $subtotal) {
        $body .= '<tr style="'.$rowSubtotal.'">';
        $body .= '<td style="'.$cellLeftAlignedSubtotal.'">'.$subtotal['name'].'</td>';
        if ($bean->getAttribute('chduration'))
          $body .= '<td style="'.$cellRightAlignedSubtotal.'">'.$subtotal['time'].'</td>';
        if ($bean->getAttribute('chcost'))
          $body .= '<td style="'.$cellRightAlignedSubtotal.'">'.$subtotal['cost'].'</td>';
        $body .= '</tr>';
      }
      $body .= '</table>';
    } else {
      // Regular report.
      
      // Print table header.
      $body .= '<table border="0" cellpadding="4" cellspacing="0" width="100%">';
      $body .= '<tr>';
      $body .= '<td style="'.$tableHeader.'">'.$i18n->getKey('label.date').'</td>';
      if ($user->canManageTeam())
        $body .= '<td style="'.$tableHeader.'">'.$i18n->getKey('label.user').'</td>';
      if ($bean->getAttribute('chclient'))
        $body .= '<td style="'.$tableHeader.'">'.$i18n->getKey('label.client').'</td>';
      if ($bean->getAttribute('chproject'))
        $body .= '<td style="'.$tableHeader.'">'.$i18n->getKey('label.project').'</td>';
      if ($bean->getAttribute('chtask'))
        $body .= '<td style="'.$tableHeader.'">'.$i18n->getKey('label.task').'</td>';
      if ($bean->getAttribute('chcf_1'))
        $body .= '<td style="'.$tableHeader.'">'.$custom_fields->fields[0]['label'].'</td>';
      if ($bean->getAttribute('chstart'))
        $body .= '<td style="'.$tableHeader.'">'.$i18n->getKey('label.start').'</td>';
      if ($bean->getAttribute('chfinish'))
        $body .= '<td style="'.$tableHeader.'">'.$i18n->getKey('label.finish').'</td>';
      if ($bean->getAttribute('chduration'))
        $body .= '<td style="'.$tableHeader.'">'.$i18n->getKey('label.duration').'</td>';
      if ($bean->getAttribute('chnote'))
        $body .= '<td style="'.$tableHeader.'">'.$i18n->getKey('label.note').'</td>';
      if ($bean->getAttribute('chcost'))
        $body .= '<td style="'.$tableHeader.'">'.$i18n->getKey('label.cost').'</td>';
      if ($bean->getAttribute('chinvoice'))
        $body .= '<td style="'.$tableHeader.'">'.$i18n->getKey('label.invoice').'</td>';
      $body .= '</tr>';
      
      // Initialize control variables to print subtotals.
      if ($report_items && 'no_grouping' != $bean->getAttribute('group_by')) {
        $print_subtotals = true;
        $first_pass = true;
        $prev_value = '';
        $cur_value = '';
      }
      
      // Print report items.
      foreach ($report_items as $record) {
      	
      	// Print a subtotal row after a block of grouped items.
      	if ($print_subtotals) {
      	  $cur_value = $record['grouped_by'];
          if ($cur_value != $prev_value && !$first_pass) {
            $body .= '<tr style="'.$rowSubtotal.'">';
            $body .= '<td style="'.$cellLeftAlignedSubtotal.'">'.$subtotals[$prev_value]['name'].'</td>';
            if ($user->canManageTeam()) $body .= '<td></td>';
            if ($bean->getAttribute('chclient')) $body .= '<td></td>';
            if ($bean->getAttribute('chproject')) $body .= '<td></td>';
            if ($bean->getAttribute('chtask')) $body .= '<td></td>';
            if ($bean->getAttribute('chcf_1')) $body .= '<td></td>';
            if ($bean->getAttribute('chstart')) $body .= '<td></td>';
            if ($bean->getAttribute('chfinish')) $body .= '<td></td>';
            if ($bean->getAttribute('chduration')) $body .= '<td style="'.$cellRightAlignedSubtotal.'">'.$subtotals[$prev_value]['time'].'</td>';
            if ($bean->getAttribute('chnote')) $body .= '<td></td>';
            if ($bean->getAttribute('chcost')) $body .= '<td style="'.$cellRightAlignedSubtotal.'">'.$subtotals[$prev_value]['cost'].'</td>';
            if ($bean->getAttribute('chinvoice')) $body .= '<td></td>';
            $body .= '</tr>';
            $body .= '<tr><td>&nbsp;</td></tr>';	
          }
          $first_pass = false;
      	}
      	
        // Print a regular row.      	
      	$body .= '<tr style="'.$rowItem.'">';
      	$body .= '<td style="'.$cellLeftAligned.'">'.$record['date'].'</td>';
      	if ($user->canManageTeam())
      	  $body .= '<td style="'.$cellLeftAligned.'">'.$record['user'].'</td>';
      	if ($bean->getAttribute('chclient'))
          $body .= '<td style="'.$cellLeftAligned.'">'.$record['client'].'</td>';
        if ($bean->getAttribute('chproject'))
          $body .= '<td style="'.$cellLeftAligned.'">'.$record['project'].'</td>';
        if ($bean->getAttribute('chtask'))
          $body .= '<t style="'.$cellLeftAligned.'"d>'.$record['task'].'</td>';
        if ($bean->getAttribute('chcf_1'))
          $body .= '<td style="'.$cellLeftAligned.'">'.$record['cf_1'].'</td>';
        if ($bean->getAttribute('chstart'))
          $body .= '<td style="'.$cellRightAligned.'">'.$record['start'].'</td>';
      	if ($bean->getAttribute('chfinish'))
          $body .= '<td style="'.$cellRightAligned.'">'.$record['finish'].'</td>';
      	if ($bean->getAttribute('chduration'))
          $body .= '<td style="'.$cellRightAligned.'">'.$record['duration'].'</td>';
      	if ($bean->getAttribute('chnote'))
          $body .= '<td style="'.$cellLeftAligned.'">'.$record['note'].'</td>';
      	if ($bean->getAttribute('chcost'))
          $body .= '<td style="'.$cellRightAligned.'">'.$record['cost'].'</td>';
       	if ($bean->getAttribute('chinvoice'))
          $body .= '<td style="'.$cellRightAligned.'">'.$record['invoice'].'</td>';                    
      	$body .= '</tr>';
      	
      	$prev_value = $record['grouped_by'];
      }
      
      // Print a terminating subtotal.
      if ($print_subtotals) {
        $body .= '<tr style="'.$rowSubtotal.'">';
        $body .= '<td style="'.$cellLeftAlignedSubtotal.'">'.$subtotals[$cur_value]['name'].'</td>';
        if ($user->canManageTeam()) $body .= '<td></td>';
        if ($bean->getAttribute('chclient')) $body .= '<td></td>';
        if ($bean->getAttribute('chproject')) $body .= '<td></td>';
        if ($bean->getAttribute('chtask')) $body .= '<td></td>';
        if ($bean->getAttribute('chcf_1')) $body .= '<td></td>';
        if ($bean->getAttribute('chstart')) $body .= '<td></td>';
        if ($bean->getAttribute('chfinish')) $body .= '<td></td>';
        if ($bean->getAttribute('chduration')) $body .= '<td style="'.$cellRightAlignedSubtotal.'">'.$subtotals[$cur_value]['time'].'</td>';
        if ($bean->getAttribute('chnote')) $body .= '<td></td>';
        if ($bean->getAttribute('chcost')) $body .= '<td style="'.$cellRightAlignedSubtotal.'">'.$subtotals[$cur_value]['cost'].'</td>';
        if ($bean->getAttribute('chinvoice')) $body .= '<td></td>';
        $body .= '</tr>';
        $body .= '<tr><td>&nbsp;</td></tr>';
      }      
      
      $body .= '</table>';
    }
    
    // Print summary.
    $body .= '<table width="720" cellspacing="4" cellpadding="4" border="0">';
    $body .= '<tr>';
    $body .= '<td align="left">';
    $body .= '<table border="0">';
    $body .= '<tr><td style="'.$cellLeftAlignedSubtotal.'">'.$i18n->getKey('label.start_date').':</td><td style="'.$cellRightAlignedSubtotal.'">'.$totals['start_date'].'</td></tr>';
    $body .= '<tr><td style="'.$cellLeftAlignedSubtotal.'">'.$i18n->getKey('label.end_date').':</td><td style="'.$cellRightAlignedSubtotal.'">'.$totals['end_date'].'</td></tr>';
    $body .= '<tr><td style="'.$cellLeftAlignedSubtotal.'">'.$i18n->getKey('label.total_hours').':</td><td style="'.$cellRightAlignedSubtotal.'">'.$totals['time'].'</td></tr>';
    if ($bean->getAttribute('chcost'))
      $body .= '<tr><td style="'.$cellLeftAlignedSubtotal.'">'.$i18n->getKey('label.total_cost').':</td><td style="'.$cellRightAlignedSubtotal.'">'.$user->currency.' '.$totals['cost'].'</td></tr>';
    $body .= '</table>';
    $body .= '</td>';
    $body .= '</tr>';
    $body .= '</table>';
  
    // Output comment.
    $body .= '<p>'.$comment.'</p>';
    // Output footer.
    $body .= '<p style="text-align: center;">'.$i18n->getKey('form.mail.footer').'</p>';

    // Finish creating email body.
    $body .= '</body></html>';

    return $body;
  }
}
?>