<?php
// +----------------------------------------------------------------------+
// | Anuko Time Tracker
// +----------------------------------------------------------------------+
// | Copyright (c) Anuko International Ltd. (http://www.anuko.com)
// +----------------------------------------------------------------------+
// | LIBERAL FREEWARE LICENSE: This source code document may be used
// | by anyone for any purpose, and freely redistributed alone or in
// | combination with other software, provided that the license is obeyed.
// |
// | There are only two ways to violate the license:
// |
// | 1. To redistribute this code in source form, with the copyright
// |    notice or license removed or altered. (Distributing in compiled
// |    forms without embedded copyright notices is permitted).
// |
// | 2. To redistribute modified versions of this code in *any* form
// |    that bears insufficient indications that the modifications are
// |    not the work of the original author(s).
// |
// | This license applies to this document only, not any other software
// | that it may be combined with.
// |
// +----------------------------------------------------------------------+
// | Contributors:
// | http://www.anuko.com/time_tracker/credits.htm
// +----------------------------------------------------------------------+

require_once('initialize.php');
import('form.Form');
import('ttTeamHelper');
import('form.check.Validator');
import('ttInvoiceHelper');

// Authentication check and redirect.
if ($auth->isAuthenticated()) {
  if ($user->isAdmin()) {
    header('Location: admin_teams.php');
    exit();
  }
} else {
  header('Location: login.php');
  exit();
}

if ($request->getMethod() == 'POST') {
  $cl_date = $request->getParameter('date');
  $cl_client = $request->getParameter('client');
  $cl_number = trim($request->getParameter('number'));
  $cl_start = $request->getParameter('start');
  $cl_finish = $request->getParameter('finish');
  $cl_discount = $request->getParameter('discount');
} else {
  $today = new DateAndTime($user->date_format);
  $cl_date = $today->toString();
}

$form = new Form('invoiceForm');
$form->addInput(array('type'=>'datefield','name'=>'date','size'=>'20','value'=>$cl_date));

// Dropdown for clients if the clients plugin is enabled.
if (in_array('cl', explode(',', $user->plugins))) {
  $clients = ttTeamHelper::getActiveClients($user->team_id);
  $form->addInput(array('type'=>'combobox','name'=>'client','style'=>'width: 250px;','data'=>$clients,'datakeys'=>array('id','name'),'value'=>$cl_client,'empty'=>array(''=>$i18n->getKey('dropdown.select_client'))));
}
$form->addInput(array('type'=>'text','maxlength'=>'100','name'=>'number','style'=>'width: 250px;','value'=>$cl_number));
$form->addInput(array('type'=>'datefield','maxlength'=>'20','name'=>'start','value'=>$cl_start));
$form->addInput(array('type'=>'datefield','maxlength'=>'20','name'=>'finish','value'=>$cl_finish));
$form->addInput(array('type'=>'floatfield','name'=>'discount','size'=>'10','format'=>'.2','value'=>$cl_discount));
$form->addInput(array('type'=>'submit','name'=>'btn_submit','value'=>$i18n->getKey('button.add')));

if ($request->getMethod() == 'POST') {
  // Validate input data.
  $validator = new Validator($cl_date);
  $validator->validateDate();
  if (!$validator->isValid()) $errors->add($i18n->getKey('error.field'), $i18n->getKey('label.date'));
	
  if (!$cl_client) $errors->add($i18n->getKey('error.client'));
	
  $validator = new Validator($cl_number);
  $validator->validateString();
  if (!$validator->isValid())
    $errors->add($i18n->getKey('error.field'), $i18n->getKey('form.invoice.number'));

  $validator = new Validator($cl_start);
  $validator->validateDate();
  if (!$validator->isValid()) $errors->add($i18n->getKey('error.field'), $i18n->getKey('label.start_date'));
    
  $validator = new Validator($cl_finish);
  $validator->validateDate();
  if (!$validator->isValid()) $errors->add($i18n->getKey('error.field'), $i18n->getKey('label.end_date'));

  if (isset($cl_discount)) {
    $validator = new Validator($cl_discount);
    $validator->validateFloat($i18n->getKey('format.float_delimiter'));
    if (!$validator->isValid())
      $errors->add($i18n->getKey('error.field'), $i18n->getKey('label.discount'));
  }
  
  $fields = array('date'=>$cl_date,'name'=>$cl_number,'client_id'=>$cl_client,'start_date'=>$cl_start,'end_date'=>$cl_finish,'discount'=>$cl_discount);
  if ($errors->isEmpty()) {
    if (ttInvoiceHelper::getInvoiceByName($cl_number))
      $errors->add($i18n->getKey('error.invoice_exists'));
      
    if (!ttInvoiceHelper::invoiceableItemsExist($fields))
      $errors->add($i18n->getKey('error.no_invoiceable_items'));    
  }

  if ($errors->isEmpty()) {
  	// Now we can go ahead and create our invoice.
    if (ttInvoiceHelper::createInvoice($fields)) {
      header('Location: invoices.php');
      exit();
    }
      $errors->add($i18n->getKey('error.db'));	
  }
} // post

$smarty->assign('forms', array($form->getName()=>$form->toArray()));
$smarty->assign('onload', "onLoad=\"document.invoiceForm.number.focus()\"");
$smarty->assign('title', $i18n->getKey('title.add_invoice'));
$smarty->assign('content_page_name', 'invoice_add.tpl');
$smarty->display('index.tpl');
?>